# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_EXTREMA_METADATA = Metadata(
    id="bde4b9ab2a53737d37cd5c4267bc185877f0a3aa.workbench",
    name="volume-extrema",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeExtremaPresmoothParamsDictNoTag = typing.TypedDict('_VolumeExtremaPresmoothParamsDictNoTag', {
    "kernel": float,
    "fwhm": bool,
})
VolumeExtremaPresmoothParamsDictTagged = typing.TypedDict('VolumeExtremaPresmoothParamsDictTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "fwhm": bool,
})
VolumeExtremaPresmoothParamsDict = _VolumeExtremaPresmoothParamsDictNoTag | VolumeExtremaPresmoothParamsDictTagged


_VolumeExtremaThresholdParamsDictNoTag = typing.TypedDict('_VolumeExtremaThresholdParamsDictNoTag', {
    "low": float,
    "high": float,
})
VolumeExtremaThresholdParamsDictTagged = typing.TypedDict('VolumeExtremaThresholdParamsDictTagged', {
    "@type": typing.Literal["threshold"],
    "low": float,
    "high": float,
})
VolumeExtremaThresholdParamsDict = _VolumeExtremaThresholdParamsDictNoTag | VolumeExtremaThresholdParamsDictTagged


_VolumeExtremaParamsDictNoTag = typing.TypedDict('_VolumeExtremaParamsDictNoTag', {
    "volume-in": InputPathType,
    "distance": float,
    "volume-out": str,
    "presmooth": typing.NotRequired[VolumeExtremaPresmoothParamsDict | None],
    "threshold": typing.NotRequired[VolumeExtremaThresholdParamsDict | None],
    "subvolume": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "only-minima": bool,
    "only-maxima": bool,
    "consolidate-mode": bool,
    "sum-subvols": bool,
})
VolumeExtremaParamsDictTagged = typing.TypedDict('VolumeExtremaParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-extrema"],
    "volume-in": InputPathType,
    "distance": float,
    "volume-out": str,
    "presmooth": typing.NotRequired[VolumeExtremaPresmoothParamsDict | None],
    "threshold": typing.NotRequired[VolumeExtremaThresholdParamsDict | None],
    "subvolume": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "only-minima": bool,
    "only-maxima": bool,
    "consolidate-mode": bool,
    "sum-subvols": bool,
})
VolumeExtremaParamsDict = _VolumeExtremaParamsDictNoTag | VolumeExtremaParamsDictTagged


def volume_extrema_presmooth(
    kernel: float,
    fwhm: bool = False,
) -> VolumeExtremaPresmoothParamsDictTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "fwhm": fwhm,
    }
    return params


def volume_extrema_presmooth_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeExtremaPresmoothParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def volume_extrema_presmooth_cargs(
    params: VolumeExtremaPresmoothParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-presmooth",
        str(params.get("kernel", None))
    ])
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def volume_extrema_threshold(
    low: float,
    high: float,
) -> VolumeExtremaThresholdParamsDictTagged:
    """
    Build parameters.
    
    Args:
        low: the largest value to consider for being a minimum.
        high: the smallest value to consider for being a maximum.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "threshold",
        "low": low,
        "high": high,
    }
    return params


def volume_extrema_threshold_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeExtremaThresholdParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("low", None) is None:
        raise StyxValidationError("`low` must not be None")
    if not isinstance(params["low"], (float, int)):
        raise StyxValidationError(f'`low` has the wrong type: Received `{type(params.get("low", None))}` expected `float`')
    if params.get("high", None) is None:
        raise StyxValidationError("`high` must not be None")
    if not isinstance(params["high"], (float, int)):
        raise StyxValidationError(f'`high` has the wrong type: Received `{type(params.get("high", None))}` expected `float`')


def volume_extrema_threshold_cargs(
    params: VolumeExtremaThresholdParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-threshold",
        str(params.get("low", None)),
        str(params.get("high", None))
    ])
    return cargs


class VolumeExtremaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeExtremaParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output extrema volume"""


def volume_extrema_params(
    volume_in: InputPathType,
    distance: float,
    volume_out: str,
    presmooth: VolumeExtremaPresmoothParamsDict | None = None,
    threshold: VolumeExtremaThresholdParamsDict | None = None,
    subvolume: str | None = None,
    roi_volume: InputPathType | None = None,
    only_minima: bool = False,
    only_maxima: bool = False,
    consolidate_mode: bool = False,
    sum_subvols: bool = False,
) -> VolumeExtremaParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: volume file to find the extrema of.
        distance: the minimum distance between identified extrema of the same\
            type.
        volume_out: the output extrema volume.
        presmooth: smooth the volume before finding extrema.
        threshold: ignore small extrema.
        subvolume: select a single subvolume to find extrema in\
            \
            the subvolume number or name.
        roi_volume: ignore values outside the selected area\
            \
            the area to find extrema in.
        only_minima: only find the minima.
        only_maxima: only find the maxima.
        consolidate_mode: use consolidation of local minima instead of a large\
            neighborhood.
        sum_subvols: output the sum of the extrema subvolumes instead of each\
            subvolume separately.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-extrema",
        "volume-in": volume_in,
        "distance": distance,
        "volume-out": volume_out,
        "only-minima": only_minima,
        "only-maxima": only_maxima,
        "consolidate-mode": consolidate_mode,
        "sum-subvols": sum_subvols,
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    if threshold is not None:
        params["threshold"] = threshold
    if subvolume is not None:
        params["subvolume"] = subvolume
    if roi_volume is not None:
        params["roi-volume"] = roi_volume
    return params


def volume_extrema_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeExtremaParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("distance", None) is None:
        raise StyxValidationError("`distance` must not be None")
    if not isinstance(params["distance"], (float, int)):
        raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("presmooth", None) is not None:
        volume_extrema_presmooth_validate(params["presmooth"])
    if params.get("threshold", None) is not None:
        volume_extrema_threshold_validate(params["threshold"])
    if params.get("subvolume", None) is not None:
        if not isinstance(params["subvolume"], str):
            raise StyxValidationError(f'`subvolume` has the wrong type: Received `{type(params.get("subvolume", None))}` expected `str | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')
    if params.get("only-minima", False) is None:
        raise StyxValidationError("`only-minima` must not be None")
    if not isinstance(params["only-minima"], bool):
        raise StyxValidationError(f'`only-minima` has the wrong type: Received `{type(params.get("only-minima", False))}` expected `bool`')
    if params.get("only-maxima", False) is None:
        raise StyxValidationError("`only-maxima` must not be None")
    if not isinstance(params["only-maxima"], bool):
        raise StyxValidationError(f'`only-maxima` has the wrong type: Received `{type(params.get("only-maxima", False))}` expected `bool`')
    if params.get("consolidate-mode", False) is None:
        raise StyxValidationError("`consolidate-mode` must not be None")
    if not isinstance(params["consolidate-mode"], bool):
        raise StyxValidationError(f'`consolidate-mode` has the wrong type: Received `{type(params.get("consolidate-mode", False))}` expected `bool`')
    if params.get("sum-subvols", False) is None:
        raise StyxValidationError("`sum-subvols` must not be None")
    if not isinstance(params["sum-subvols"], bool):
        raise StyxValidationError(f'`sum-subvols` has the wrong type: Received `{type(params.get("sum-subvols", False))}` expected `bool`')


def volume_extrema_cargs(
    params: VolumeExtremaParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-extrema"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(str(params.get("distance", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("presmooth", None) is not None or params.get("threshold", None) is not None:
        cargs.extend([
            *(volume_extrema_presmooth_cargs(params.get("presmooth", None), execution) if (params.get("presmooth", None) is not None) else []),
            *(volume_extrema_threshold_cargs(params.get("threshold", None), execution) if (params.get("threshold", None) is not None) else [])
        ])
    if params.get("subvolume", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvolume", None)
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    if params.get("only-minima", False):
        cargs.append("-only-minima")
    if params.get("only-maxima", False):
        cargs.append("-only-maxima")
    if params.get("consolidate-mode", False):
        cargs.append("-consolidate-mode")
    if params.get("sum-subvols", False):
        cargs.append("-sum-subvols")
    return cargs


def volume_extrema_outputs(
    params: VolumeExtremaParamsDict,
    execution: Execution,
) -> VolumeExtremaOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeExtremaOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_extrema_execute(
    params: VolumeExtremaParamsDict,
    runner: Runner | None = None,
) -> VolumeExtremaOutputs:
    """
    FIND EXTREMA IN A VOLUME FILE.
    
    Finds extrema in a volume file, such that no two extrema of the same type
    are within <distance> of each other. The extrema are labeled as -1 for
    minima, 1 for maxima, 0 otherwise. If -only-maxima or -only-minima is
    specified, then it will ignore extrema not of the specified type. These
    options are mutually exclusive.
    
    If -sum-subvols is specified, these extrema subvolumes are summed, and the
    output has a single subvolume with this result.
    
    By default, a datapoint is an extrema only if it is more extreme than every
    other datapoint that is within <distance> from it. If -consolidate-mode is
    used, it instead starts by finding all datapoints that are more extreme than
    their immediate neighbors, then while there are any extrema within
    <distance> of each other, take the two extrema closest to each other and
    merge them into one by a weighted average based on how many original extrema
    have been merged into each.
    
    By default, all input subvolumes are used with no smoothing, use -subvolume
    to specify a single subvolume to use, and -presmooth to smooth the input
    before finding the extrema.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeExtremaOutputs`).
    """
    volume_extrema_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_EXTREMA_METADATA)
    params = execution.params(params)
    cargs = volume_extrema_cargs(params, execution)
    ret = volume_extrema_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_extrema(
    volume_in: InputPathType,
    distance: float,
    volume_out: str,
    presmooth: VolumeExtremaPresmoothParamsDict | None = None,
    threshold: VolumeExtremaThresholdParamsDict | None = None,
    subvolume: str | None = None,
    roi_volume: InputPathType | None = None,
    only_minima: bool = False,
    only_maxima: bool = False,
    consolidate_mode: bool = False,
    sum_subvols: bool = False,
    runner: Runner | None = None,
) -> VolumeExtremaOutputs:
    """
    FIND EXTREMA IN A VOLUME FILE.
    
    Finds extrema in a volume file, such that no two extrema of the same type
    are within <distance> of each other. The extrema are labeled as -1 for
    minima, 1 for maxima, 0 otherwise. If -only-maxima or -only-minima is
    specified, then it will ignore extrema not of the specified type. These
    options are mutually exclusive.
    
    If -sum-subvols is specified, these extrema subvolumes are summed, and the
    output has a single subvolume with this result.
    
    By default, a datapoint is an extrema only if it is more extreme than every
    other datapoint that is within <distance> from it. If -consolidate-mode is
    used, it instead starts by finding all datapoints that are more extreme than
    their immediate neighbors, then while there are any extrema within
    <distance> of each other, take the two extrema closest to each other and
    merge them into one by a weighted average based on how many original extrema
    have been merged into each.
    
    By default, all input subvolumes are used with no smoothing, use -subvolume
    to specify a single subvolume to use, and -presmooth to smooth the input
    before finding the extrema.
    
    Args:
        volume_in: volume file to find the extrema of.
        distance: the minimum distance between identified extrema of the same\
            type.
        volume_out: the output extrema volume.
        presmooth: smooth the volume before finding extrema.
        threshold: ignore small extrema.
        subvolume: select a single subvolume to find extrema in\
            \
            the subvolume number or name.
        roi_volume: ignore values outside the selected area\
            \
            the area to find extrema in.
        only_minima: only find the minima.
        only_maxima: only find the maxima.
        consolidate_mode: use consolidation of local minima instead of a large\
            neighborhood.
        sum_subvols: output the sum of the extrema subvolumes instead of each\
            subvolume separately.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeExtremaOutputs`).
    """
    params = volume_extrema_params(
        volume_in=volume_in,
        distance=distance,
        volume_out=volume_out,
        presmooth=presmooth,
        threshold=threshold,
        subvolume=subvolume,
        roi_volume=roi_volume,
        only_minima=only_minima,
        only_maxima=only_maxima,
        consolidate_mode=consolidate_mode,
        sum_subvols=sum_subvols,
    )
    return volume_extrema_execute(params, runner)


__all__ = [
    "VOLUME_EXTREMA_METADATA",
    "VolumeExtremaOutputs",
    "VolumeExtremaParamsDict",
    "VolumeExtremaParamsDictTagged",
    "VolumeExtremaPresmoothParamsDict",
    "VolumeExtremaPresmoothParamsDictTagged",
    "VolumeExtremaThresholdParamsDict",
    "VolumeExtremaThresholdParamsDictTagged",
    "volume_extrema",
    "volume_extrema_execute",
    "volume_extrema_params",
    "volume_extrema_presmooth",
    "volume_extrema_threshold",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_ROIS_FROM_EXTREMA_METADATA = Metadata(
    id="9741081a1f9bdfe5deb553d7d2cb39c92eb6cafe.workbench",
    name="volume-rois-from-extrema",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeRoisFromExtremaParamsDictNoTag = typing.TypedDict('_VolumeRoisFromExtremaParamsDictNoTag', {
    "volume-in": InputPathType,
    "limit": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "method": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "sigma": typing.NotRequired[float | None],
})
VolumeRoisFromExtremaParamsDictTagged = typing.TypedDict('VolumeRoisFromExtremaParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-rois-from-extrema"],
    "volume-in": InputPathType,
    "limit": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "method": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "sigma": typing.NotRequired[float | None],
})
VolumeRoisFromExtremaParamsDict = _VolumeRoisFromExtremaParamsDictNoTag | VolumeRoisFromExtremaParamsDictTagged


class VolumeRoisFromExtremaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeRoisFromExtremaParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_rois_from_extrema_params(
    volume_in: InputPathType,
    limit: float,
    volume_out: str,
    subvol: str | None = None,
    method: str | None = None,
    roi_volume: InputPathType | None = None,
    sigma: float | None = None,
) -> VolumeRoisFromExtremaParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        limit: distance limit from voxel center, in mm.
        volume_out: the output volume.
        subvol: select a single subvolume to take the gradient of\
            \
            the subvolume number or name.
        method: how to handle overlapping ROIs, default ALLOW\
            \
            the method of resolving overlaps.
        roi_volume: select a region of interest to use\
            \
            the region to use.
        sigma: generate a gaussian kernel instead of a flat ROI\
            \
            the sigma for the gaussian kernel, in mm.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-rois-from-extrema",
        "volume-in": volume_in,
        "limit": limit,
        "volume-out": volume_out,
    }
    if subvol is not None:
        params["subvol"] = subvol
    if method is not None:
        params["method"] = method
    if roi_volume is not None:
        params["roi-volume"] = roi_volume
    if sigma is not None:
        params["sigma"] = sigma
    return params


def volume_rois_from_extrema_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeRoisFromExtremaParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("limit", None) is None:
        raise StyxValidationError("`limit` must not be None")
    if not isinstance(params["limit"], (float, int)):
        raise StyxValidationError(f'`limit` has the wrong type: Received `{type(params.get("limit", None))}` expected `float`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("method", None) is not None:
        if not isinstance(params["method"], str):
            raise StyxValidationError(f'`method` has the wrong type: Received `{type(params.get("method", None))}` expected `str | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')
    if params.get("sigma", None) is not None:
        if not isinstance(params["sigma"], (float, int)):
            raise StyxValidationError(f'`sigma` has the wrong type: Received `{type(params.get("sigma", None))}` expected `float | None`')


def volume_rois_from_extrema_cargs(
    params: VolumeRoisFromExtremaParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-rois-from-extrema"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(str(params.get("limit", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("method", None) is not None:
        cargs.extend([
            "-overlap-logic",
            params.get("method", None)
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    if params.get("sigma", None) is not None:
        cargs.extend([
            "-gaussian",
            str(params.get("sigma", None))
        ])
    return cargs


def volume_rois_from_extrema_outputs(
    params: VolumeRoisFromExtremaParamsDict,
    execution: Execution,
) -> VolumeRoisFromExtremaOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeRoisFromExtremaOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_rois_from_extrema_execute(
    params: VolumeRoisFromExtremaParamsDict,
    runner: Runner | None = None,
) -> VolumeRoisFromExtremaOutputs:
    """
    CREATE VOLUME ROI MAPS FROM EXTREMA MAPS.
    
    For each nonzero value in each map, make a map with an ROI around that
    location. If the -gaussian option is specified, then normalized gaussian
    kernels are output instead of ROIs. The <method> argument to -overlap-logic
    must be one of ALLOW, CLOSEST, or EXCLUDE. ALLOW is the default, and means
    that ROIs are treated independently and may overlap. CLOSEST means that ROIs
    may not overlap, and that no ROI contains vertices that are closer to a
    different seed vertex. EXCLUDE means that ROIs may not overlap, and that any
    vertex within range of more than one ROI does not belong to any ROI.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeRoisFromExtremaOutputs`).
    """
    volume_rois_from_extrema_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_ROIS_FROM_EXTREMA_METADATA)
    params = execution.params(params)
    cargs = volume_rois_from_extrema_cargs(params, execution)
    ret = volume_rois_from_extrema_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_rois_from_extrema(
    volume_in: InputPathType,
    limit: float,
    volume_out: str,
    subvol: str | None = None,
    method: str | None = None,
    roi_volume: InputPathType | None = None,
    sigma: float | None = None,
    runner: Runner | None = None,
) -> VolumeRoisFromExtremaOutputs:
    """
    CREATE VOLUME ROI MAPS FROM EXTREMA MAPS.
    
    For each nonzero value in each map, make a map with an ROI around that
    location. If the -gaussian option is specified, then normalized gaussian
    kernels are output instead of ROIs. The <method> argument to -overlap-logic
    must be one of ALLOW, CLOSEST, or EXCLUDE. ALLOW is the default, and means
    that ROIs are treated independently and may overlap. CLOSEST means that ROIs
    may not overlap, and that no ROI contains vertices that are closer to a
    different seed vertex. EXCLUDE means that ROIs may not overlap, and that any
    vertex within range of more than one ROI does not belong to any ROI.
    
    Args:
        volume_in: the input volume.
        limit: distance limit from voxel center, in mm.
        volume_out: the output volume.
        subvol: select a single subvolume to take the gradient of\
            \
            the subvolume number or name.
        method: how to handle overlapping ROIs, default ALLOW\
            \
            the method of resolving overlaps.
        roi_volume: select a region of interest to use\
            \
            the region to use.
        sigma: generate a gaussian kernel instead of a flat ROI\
            \
            the sigma for the gaussian kernel, in mm.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeRoisFromExtremaOutputs`).
    """
    params = volume_rois_from_extrema_params(
        volume_in=volume_in,
        limit=limit,
        volume_out=volume_out,
        subvol=subvol,
        method=method,
        roi_volume=roi_volume,
        sigma=sigma,
    )
    return volume_rois_from_extrema_execute(params, runner)


__all__ = [
    "VOLUME_ROIS_FROM_EXTREMA_METADATA",
    "VolumeRoisFromExtremaOutputs",
    "VolumeRoisFromExtremaParamsDict",
    "VolumeRoisFromExtremaParamsDictTagged",
    "volume_rois_from_extrema",
    "volume_rois_from_extrema_execute",
    "volume_rois_from_extrema_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_MERGE_METADATA = Metadata(
    id="58734bb6d294ee25611f347ddc44d2a3b041b696.workbench",
    name="metric-merge",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricMergeUpToParamsDictNoTag = typing.TypedDict('_MetricMergeUpToParamsDictNoTag', {
    "last-column": str,
    "reverse": bool,
})
MetricMergeUpToParamsDictTagged = typing.TypedDict('MetricMergeUpToParamsDictTagged', {
    "@type": typing.Literal["up-to"],
    "last-column": str,
    "reverse": bool,
})
MetricMergeUpToParamsDict = _MetricMergeUpToParamsDictNoTag | MetricMergeUpToParamsDictTagged


_MetricMergeColumnParamsDictNoTag = typing.TypedDict('_MetricMergeColumnParamsDictNoTag', {
    "column": str,
    "up-to": typing.NotRequired[MetricMergeUpToParamsDict | None],
})
MetricMergeColumnParamsDictTagged = typing.TypedDict('MetricMergeColumnParamsDictTagged', {
    "@type": typing.Literal["column"],
    "column": str,
    "up-to": typing.NotRequired[MetricMergeUpToParamsDict | None],
})
MetricMergeColumnParamsDict = _MetricMergeColumnParamsDictNoTag | MetricMergeColumnParamsDictTagged


_MetricMergeMetricParamsDictNoTag = typing.TypedDict('_MetricMergeMetricParamsDictNoTag', {
    "metric-in": InputPathType,
    "column": typing.NotRequired[list[MetricMergeColumnParamsDict] | None],
})
MetricMergeMetricParamsDictTagged = typing.TypedDict('MetricMergeMetricParamsDictTagged', {
    "@type": typing.Literal["metric"],
    "metric-in": InputPathType,
    "column": typing.NotRequired[list[MetricMergeColumnParamsDict] | None],
})
MetricMergeMetricParamsDict = _MetricMergeMetricParamsDictNoTag | MetricMergeMetricParamsDictTagged


_MetricMergeParamsDictNoTag = typing.TypedDict('_MetricMergeParamsDictNoTag', {
    "metric-out": str,
    "metric": typing.NotRequired[list[MetricMergeMetricParamsDict] | None],
})
MetricMergeParamsDictTagged = typing.TypedDict('MetricMergeParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-merge"],
    "metric-out": str,
    "metric": typing.NotRequired[list[MetricMergeMetricParamsDict] | None],
})
MetricMergeParamsDict = _MetricMergeParamsDictNoTag | MetricMergeParamsDictTagged


def metric_merge_up_to(
    last_column: str,
    reverse: bool = False,
) -> MetricMergeUpToParamsDictTagged:
    """
    Build parameters.
    
    Args:
        last_column: the number or name of the last column to include.
        reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up-to",
        "last-column": last_column,
        "reverse": reverse,
    }
    return params


def metric_merge_up_to_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricMergeUpToParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("last-column", None) is None:
        raise StyxValidationError("`last-column` must not be None")
    if not isinstance(params["last-column"], str):
        raise StyxValidationError(f'`last-column` has the wrong type: Received `{type(params.get("last-column", None))}` expected `str`')
    if params.get("reverse", False) is None:
        raise StyxValidationError("`reverse` must not be None")
    if not isinstance(params["reverse"], bool):
        raise StyxValidationError(f'`reverse` has the wrong type: Received `{type(params.get("reverse", False))}` expected `bool`')


def metric_merge_up_to_cargs(
    params: MetricMergeUpToParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-up-to",
        params.get("last-column", None)
    ])
    if params.get("reverse", False):
        cargs.append("-reverse")
    return cargs


def metric_merge_column(
    column: str,
    up_to: MetricMergeUpToParamsDict | None = None,
) -> MetricMergeColumnParamsDictTagged:
    """
    Build parameters.
    
    Args:
        column: the column number or name.
        up_to: use an inclusive range of columns.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "column",
        "column": column,
    }
    if up_to is not None:
        params["up-to"] = up_to
    return params


def metric_merge_column_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricMergeColumnParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("column", None) is None:
        raise StyxValidationError("`column` must not be None")
    if not isinstance(params["column"], str):
        raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str`')
    if params.get("up-to", None) is not None:
        metric_merge_up_to_validate(params["up-to"])


def metric_merge_column_cargs(
    params: MetricMergeColumnParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-column",
        params.get("column", None),
        *(metric_merge_up_to_cargs(params.get("up-to", None), execution) if (params.get("up-to", None) is not None) else [])
    ])
    return cargs


def metric_merge_metric(
    metric_in: InputPathType,
    column: list[MetricMergeColumnParamsDict] | None = None,
) -> MetricMergeMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric_in: a metric file to use columns from.
        column: select a single column to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "metric",
        "metric-in": metric_in,
    }
    if column is not None:
        params["column"] = column
    return params


def metric_merge_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricMergeMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], list):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `list[MetricMergeColumnParamsDict] | None`')
        for e in params["column"]:
            metric_merge_column_validate(e)


def metric_merge_metric_cargs(
    params: MetricMergeMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-metric",
        execution.input_file(params.get("metric-in", None)),
        *([a for c in [metric_merge_column_cargs(s, execution) for s in params.get("column", None)] for a in c] if (params.get("column", None) is not None) else [])
    ])
    return cargs


class MetricMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricMergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_merge_params(
    metric_out: str,
    metric: list[MetricMergeMetricParamsDict] | None = None,
) -> MetricMergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric_out: the output metric.
        metric: specify an input metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-merge",
        "metric-out": metric_out,
    }
    if metric is not None:
        params["metric"] = metric
    return params


def metric_merge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricMergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("metric", None) is not None:
        if not isinstance(params["metric"], list):
            raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `list[MetricMergeMetricParamsDict] | None`')
        for e in params["metric"]:
            metric_merge_metric_validate(e)


def metric_merge_cargs(
    params: MetricMergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-merge"
    ])
    cargs.append(params.get("metric-out", None))
    if params.get("metric", None) is not None:
        cargs.extend([a for c in [metric_merge_metric_cargs(s, execution) for s in params.get("metric", None)] for a in c])
    return cargs


def metric_merge_outputs(
    params: MetricMergeParamsDict,
    execution: Execution,
) -> MetricMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricMergeOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_merge_execute(
    params: MetricMergeParamsDict,
    runner: Runner | None = None,
) -> MetricMergeOutputs:
    """
    MERGE METRIC FILES INTO A NEW FILE.
    
    Takes one or more metric files and constructs a new metric file by
    concatenating columns from them. The input metric files must have the same
    number of vertices and same structure.
    
    Example: wb_command -metric-merge out.func.gii -metric first.func.gii
    -column 1 -metric second.func.gii
    
    This example would take the first column from first.func.gii, followed by
    all columns from second.func.gii, and write these columns to out.func.gii.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricMergeOutputs`).
    """
    metric_merge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_MERGE_METADATA)
    params = execution.params(params)
    cargs = metric_merge_cargs(params, execution)
    ret = metric_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_merge(
    metric_out: str,
    metric: list[MetricMergeMetricParamsDict] | None = None,
    runner: Runner | None = None,
) -> MetricMergeOutputs:
    """
    MERGE METRIC FILES INTO A NEW FILE.
    
    Takes one or more metric files and constructs a new metric file by
    concatenating columns from them. The input metric files must have the same
    number of vertices and same structure.
    
    Example: wb_command -metric-merge out.func.gii -metric first.func.gii
    -column 1 -metric second.func.gii
    
    This example would take the first column from first.func.gii, followed by
    all columns from second.func.gii, and write these columns to out.func.gii.
    
    Args:
        metric_out: the output metric.
        metric: specify an input metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricMergeOutputs`).
    """
    params = metric_merge_params(
        metric_out=metric_out,
        metric=metric,
    )
    return metric_merge_execute(params, runner)


__all__ = [
    "METRIC_MERGE_METADATA",
    "MetricMergeColumnParamsDict",
    "MetricMergeColumnParamsDictTagged",
    "MetricMergeMetricParamsDict",
    "MetricMergeMetricParamsDictTagged",
    "MetricMergeOutputs",
    "MetricMergeParamsDict",
    "MetricMergeParamsDictTagged",
    "MetricMergeUpToParamsDict",
    "MetricMergeUpToParamsDictTagged",
    "metric_merge",
    "metric_merge_column",
    "metric_merge_execute",
    "metric_merge_metric",
    "metric_merge_params",
    "metric_merge_up_to",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FILE_INFORMATION_METADATA = Metadata(
    id="c64f1cc09da72b367adf8dd817016952eac8d776.workbench",
    name="file-information",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_FileInformationOnlyMetadataParamsDictNoTag = typing.TypedDict('_FileInformationOnlyMetadataParamsDictNoTag', {
    "key": typing.NotRequired[str | None],
})
FileInformationOnlyMetadataParamsDictTagged = typing.TypedDict('FileInformationOnlyMetadataParamsDictTagged', {
    "@type": typing.Literal["only-metadata"],
    "key": typing.NotRequired[str | None],
})
FileInformationOnlyMetadataParamsDict = _FileInformationOnlyMetadataParamsDictNoTag | FileInformationOnlyMetadataParamsDictTagged


_FileInformationParamsDictNoTag = typing.TypedDict('_FileInformationParamsDictNoTag', {
    "data-file": str,
    "only-metadata": typing.NotRequired[FileInformationOnlyMetadataParamsDict | None],
    "czi-xml": bool,
    "czi-all-sub-blocks": bool,
    "czi": bool,
    "only-cifti-xml": bool,
    "only-map-names": bool,
    "only-number-of-maps": bool,
    "only-step-interval": bool,
    "no-map-info": bool,
})
FileInformationParamsDictTagged = typing.TypedDict('FileInformationParamsDictTagged', {
    "@type": typing.Literal["workbench/file-information"],
    "data-file": str,
    "only-metadata": typing.NotRequired[FileInformationOnlyMetadataParamsDict | None],
    "czi-xml": bool,
    "czi-all-sub-blocks": bool,
    "czi": bool,
    "only-cifti-xml": bool,
    "only-map-names": bool,
    "only-number-of-maps": bool,
    "only-step-interval": bool,
    "no-map-info": bool,
})
FileInformationParamsDict = _FileInformationParamsDictNoTag | FileInformationParamsDictTagged


def file_information_only_metadata(
    key: str | None = None,
) -> FileInformationOnlyMetadataParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: only print the metadata for one key, with no formatting\
            \
            the metadata key.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "only-metadata",
    }
    if key is not None:
        params["key"] = key
    return params


def file_information_only_metadata_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FileInformationOnlyMetadataParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is not None:
        if not isinstance(params["key"], str):
            raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str | None`')


def file_information_only_metadata_cargs(
    params: FileInformationOnlyMetadataParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-only-metadata")
    if params.get("key", None) is not None:
        cargs.extend([
            "-key",
            params.get("key", None)
        ])
    return cargs


class FileInformationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FileInformationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def file_information_params(
    data_file: str,
    only_metadata: FileInformationOnlyMetadataParamsDict | None = None,
    czi_xml: bool = False,
    czi_all_sub_blocks: bool = False,
    czi: bool = False,
    only_cifti_xml: bool = False,
    only_map_names: bool = False,
    only_number_of_maps: bool = False,
    only_step_interval: bool = False,
    no_map_info: bool = False,
) -> FileInformationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        data_file: data file.
        only_metadata: suppress normal output, print file metadata.
        czi_xml: show XML from CZI file.
        czi_all_sub_blocks: show all sub-blocks in CZI file (may produce long\
            output).
        czi: For a CZI file, show information from the libCZI Info Command\
            instead of the Workbench CZI File.
        only_cifti_xml: suppress normal output, print the cifti xml if the file\
            type has it.
        only_map_names: suppress normal output, print the names of all maps.
        only_number_of_maps: suppress normal output, print the number of maps.
        only_step_interval: suppress normal output, print the interval between\
            maps.
        no_map_info: do not show map information for files that support maps.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/file-information",
        "data-file": data_file,
        "czi-xml": czi_xml,
        "czi-all-sub-blocks": czi_all_sub_blocks,
        "czi": czi,
        "only-cifti-xml": only_cifti_xml,
        "only-map-names": only_map_names,
        "only-number-of-maps": only_number_of_maps,
        "only-step-interval": only_step_interval,
        "no-map-info": no_map_info,
    }
    if only_metadata is not None:
        params["only-metadata"] = only_metadata
    return params


def file_information_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FileInformationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("data-file", None) is None:
        raise StyxValidationError("`data-file` must not be None")
    if not isinstance(params["data-file"], str):
        raise StyxValidationError(f'`data-file` has the wrong type: Received `{type(params.get("data-file", None))}` expected `str`')
    if params.get("only-metadata", None) is not None:
        file_information_only_metadata_validate(params["only-metadata"])
    if params.get("czi-xml", False) is None:
        raise StyxValidationError("`czi-xml` must not be None")
    if not isinstance(params["czi-xml"], bool):
        raise StyxValidationError(f'`czi-xml` has the wrong type: Received `{type(params.get("czi-xml", False))}` expected `bool`')
    if params.get("czi-all-sub-blocks", False) is None:
        raise StyxValidationError("`czi-all-sub-blocks` must not be None")
    if not isinstance(params["czi-all-sub-blocks"], bool):
        raise StyxValidationError(f'`czi-all-sub-blocks` has the wrong type: Received `{type(params.get("czi-all-sub-blocks", False))}` expected `bool`')
    if params.get("czi", False) is None:
        raise StyxValidationError("`czi` must not be None")
    if not isinstance(params["czi"], bool):
        raise StyxValidationError(f'`czi` has the wrong type: Received `{type(params.get("czi", False))}` expected `bool`')
    if params.get("only-cifti-xml", False) is None:
        raise StyxValidationError("`only-cifti-xml` must not be None")
    if not isinstance(params["only-cifti-xml"], bool):
        raise StyxValidationError(f'`only-cifti-xml` has the wrong type: Received `{type(params.get("only-cifti-xml", False))}` expected `bool`')
    if params.get("only-map-names", False) is None:
        raise StyxValidationError("`only-map-names` must not be None")
    if not isinstance(params["only-map-names"], bool):
        raise StyxValidationError(f'`only-map-names` has the wrong type: Received `{type(params.get("only-map-names", False))}` expected `bool`')
    if params.get("only-number-of-maps", False) is None:
        raise StyxValidationError("`only-number-of-maps` must not be None")
    if not isinstance(params["only-number-of-maps"], bool):
        raise StyxValidationError(f'`only-number-of-maps` has the wrong type: Received `{type(params.get("only-number-of-maps", False))}` expected `bool`')
    if params.get("only-step-interval", False) is None:
        raise StyxValidationError("`only-step-interval` must not be None")
    if not isinstance(params["only-step-interval"], bool):
        raise StyxValidationError(f'`only-step-interval` has the wrong type: Received `{type(params.get("only-step-interval", False))}` expected `bool`')
    if params.get("no-map-info", False) is None:
        raise StyxValidationError("`no-map-info` must not be None")
    if not isinstance(params["no-map-info"], bool):
        raise StyxValidationError(f'`no-map-info` has the wrong type: Received `{type(params.get("no-map-info", False))}` expected `bool`')


def file_information_cargs(
    params: FileInformationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-file-information"
    ])
    cargs.append(params.get("data-file", None))
    if params.get("only-metadata", None) is not None:
        cargs.extend(file_information_only_metadata_cargs(params.get("only-metadata", None), execution))
    if params.get("czi-xml", False):
        cargs.append("-czi-xml")
    if params.get("czi-all-sub-blocks", False):
        cargs.append("-czi-all-sub-blocks")
    if params.get("czi", False):
        cargs.append("-czi")
    if params.get("only-cifti-xml", False):
        cargs.append("-only-cifti-xml")
    if params.get("only-map-names", False):
        cargs.append("-only-map-names")
    if params.get("only-number-of-maps", False):
        cargs.append("-only-number-of-maps")
    if params.get("only-step-interval", False):
        cargs.append("-only-step-interval")
    if params.get("no-map-info", False):
        cargs.append("-no-map-info")
    return cargs


def file_information_outputs(
    params: FileInformationParamsDict,
    execution: Execution,
) -> FileInformationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FileInformationOutputs(
        root=execution.output_file("."),
    )
    return ret


def file_information_execute(
    params: FileInformationParamsDict,
    runner: Runner | None = None,
) -> FileInformationOutputs:
    """
    LIST INFORMATION ABOUT A FILE'S CONTENT.
    
    List information about the content of a data file. Only one -only option may
    be specified. The information listed when no -only option is present is
    dependent upon the type of data file.
    
    Library paths:
    /usr/lib/x86_64-linux-gnu/qt5/plugins
    /workspace/source/build/CommandLine
    
    
    File and extensions for reading and writing:
    Annotation: wb_annot
    Annotation Text Substitution: wb_annsub.csv
    Border: border, wb_border
    CIFTI - Dense: dconn.nii
    CIFTI - Dense Label: dlabel.nii
    CIFTI - Dense Parcel: dpconn.nii
    CIFTI - Dense Scalar: dscalar.nii
    CIFTI - Dense Data Series: dtseries.nii
    CIFTI - Fiber Orientations TEMPORARY: fiberTEMP.nii
    CIFTI - Fiber Trajectory TEMPORARY: trajTEMP.wbsparse
    CIFTI - Fiber Trajectory Maps: trajMaps.wbsparse
    CIFTI - Parcel: pconn.nii
    CIFTI - Parcel Dense: pdconn.nii
    CIFTI - Parcel Label: plabel.nii
    CIFTI - Parcel Scalar: pscalar.nii
    CIFTI - Parcel Series: ptseries.nii
    CIFTI - Scalar Data Series: sdseries.nii
    CZI Image: czi
    Foci: foci, wb_foci
    Histology Slices: metaczi, meta-image
    Image Read: bmp, gif, jpeg, jpg, png, ppm
    Write: bmp, jpeg, jpg, png, ppm
    Label: label.gii
    Metric: func.gii, shape.gii
    OME-ZARR Image File: ome.zarr, ome.zarr.zip
    Palette: palette, wb_palette
    RGBA: rgba.gii
    Samples: wb_samples
    Scene: scene, wb_scene
    Specification: spec, wb_spec
    Surface: surf.gii
    Volume: nii, nii.gz
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FileInformationOutputs`).
    """
    file_information_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FILE_INFORMATION_METADATA)
    params = execution.params(params)
    cargs = file_information_cargs(params, execution)
    ret = file_information_outputs(params, execution)
    execution.run(cargs)
    return ret


def file_information(
    data_file: str,
    only_metadata: FileInformationOnlyMetadataParamsDict | None = None,
    czi_xml: bool = False,
    czi_all_sub_blocks: bool = False,
    czi: bool = False,
    only_cifti_xml: bool = False,
    only_map_names: bool = False,
    only_number_of_maps: bool = False,
    only_step_interval: bool = False,
    no_map_info: bool = False,
    runner: Runner | None = None,
) -> FileInformationOutputs:
    """
    LIST INFORMATION ABOUT A FILE'S CONTENT.
    
    List information about the content of a data file. Only one -only option may
    be specified. The information listed when no -only option is present is
    dependent upon the type of data file.
    
    Library paths:
    /usr/lib/x86_64-linux-gnu/qt5/plugins
    /workspace/source/build/CommandLine
    
    
    File and extensions for reading and writing:
    Annotation: wb_annot
    Annotation Text Substitution: wb_annsub.csv
    Border: border, wb_border
    CIFTI - Dense: dconn.nii
    CIFTI - Dense Label: dlabel.nii
    CIFTI - Dense Parcel: dpconn.nii
    CIFTI - Dense Scalar: dscalar.nii
    CIFTI - Dense Data Series: dtseries.nii
    CIFTI - Fiber Orientations TEMPORARY: fiberTEMP.nii
    CIFTI - Fiber Trajectory TEMPORARY: trajTEMP.wbsparse
    CIFTI - Fiber Trajectory Maps: trajMaps.wbsparse
    CIFTI - Parcel: pconn.nii
    CIFTI - Parcel Dense: pdconn.nii
    CIFTI - Parcel Label: plabel.nii
    CIFTI - Parcel Scalar: pscalar.nii
    CIFTI - Parcel Series: ptseries.nii
    CIFTI - Scalar Data Series: sdseries.nii
    CZI Image: czi
    Foci: foci, wb_foci
    Histology Slices: metaczi, meta-image
    Image Read: bmp, gif, jpeg, jpg, png, ppm
    Write: bmp, jpeg, jpg, png, ppm
    Label: label.gii
    Metric: func.gii, shape.gii
    OME-ZARR Image File: ome.zarr, ome.zarr.zip
    Palette: palette, wb_palette
    RGBA: rgba.gii
    Samples: wb_samples
    Scene: scene, wb_scene
    Specification: spec, wb_spec
    Surface: surf.gii
    Volume: nii, nii.gz
    .
    
    Args:
        data_file: data file.
        only_metadata: suppress normal output, print file metadata.
        czi_xml: show XML from CZI file.
        czi_all_sub_blocks: show all sub-blocks in CZI file (may produce long\
            output).
        czi: For a CZI file, show information from the libCZI Info Command\
            instead of the Workbench CZI File.
        only_cifti_xml: suppress normal output, print the cifti xml if the file\
            type has it.
        only_map_names: suppress normal output, print the names of all maps.
        only_number_of_maps: suppress normal output, print the number of maps.
        only_step_interval: suppress normal output, print the interval between\
            maps.
        no_map_info: do not show map information for files that support maps.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FileInformationOutputs`).
    """
    params = file_information_params(
        data_file=data_file,
        only_metadata=only_metadata,
        czi_xml=czi_xml,
        czi_all_sub_blocks=czi_all_sub_blocks,
        czi=czi,
        only_cifti_xml=only_cifti_xml,
        only_map_names=only_map_names,
        only_number_of_maps=only_number_of_maps,
        only_step_interval=only_step_interval,
        no_map_info=no_map_info,
    )
    return file_information_execute(params, runner)


__all__ = [
    "FILE_INFORMATION_METADATA",
    "FileInformationOnlyMetadataParamsDict",
    "FileInformationOnlyMetadataParamsDictTagged",
    "FileInformationOutputs",
    "FileInformationParamsDict",
    "FileInformationParamsDictTagged",
    "file_information",
    "file_information_execute",
    "file_information_only_metadata",
    "file_information_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_TRANSPOSE_METADATA = Metadata(
    id="a38fc279857d9723961ff9db61fed1fba00e53c2.workbench",
    name="cifti-transpose",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiTransposeParamsDictNoTag = typing.TypedDict('_CiftiTransposeParamsDictNoTag', {
    "cifti-in": InputPathType,
    "cifti-out": str,
    "limit-GB": typing.NotRequired[float | None],
})
CiftiTransposeParamsDictTagged = typing.TypedDict('CiftiTransposeParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-transpose"],
    "cifti-in": InputPathType,
    "cifti-out": str,
    "limit-GB": typing.NotRequired[float | None],
})
CiftiTransposeParamsDict = _CiftiTransposeParamsDictNoTag | CiftiTransposeParamsDictTagged


class CiftiTransposeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiTransposeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_transpose_params(
    cifti_in: InputPathType,
    cifti_out: str,
    limit_gb: float | None = None,
) -> CiftiTransposeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        cifti_out: the output cifti file.
        limit_gb: restrict memory usage\
            \
            memory limit in gigabytes.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-transpose",
        "cifti-in": cifti_in,
        "cifti-out": cifti_out,
    }
    if limit_gb is not None:
        params["limit-GB"] = limit_gb
    return params


def cifti_transpose_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiTransposeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("limit-GB", None) is not None:
        if not isinstance(params["limit-GB"], (float, int)):
            raise StyxValidationError(f'`limit-GB` has the wrong type: Received `{type(params.get("limit-GB", None))}` expected `float | None`')


def cifti_transpose_cargs(
    params: CiftiTransposeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-transpose"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(params.get("cifti-out", None))
    if params.get("limit-GB", None) is not None:
        cargs.extend([
            "-mem-limit",
            str(params.get("limit-GB", None))
        ])
    return cargs


def cifti_transpose_outputs(
    params: CiftiTransposeParamsDict,
    execution: Execution,
) -> CiftiTransposeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiTransposeOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_transpose_execute(
    params: CiftiTransposeParamsDict,
    runner: Runner | None = None,
) -> CiftiTransposeOutputs:
    """
    TRANSPOSE A CIFTI FILE.
    
    The input must be a 2-dimensional cifti file. The output is a cifti file
    where every row in the input is a column in the output.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiTransposeOutputs`).
    """
    cifti_transpose_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_TRANSPOSE_METADATA)
    params = execution.params(params)
    cargs = cifti_transpose_cargs(params, execution)
    ret = cifti_transpose_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_transpose(
    cifti_in: InputPathType,
    cifti_out: str,
    limit_gb: float | None = None,
    runner: Runner | None = None,
) -> CiftiTransposeOutputs:
    """
    TRANSPOSE A CIFTI FILE.
    
    The input must be a 2-dimensional cifti file. The output is a cifti file
    where every row in the input is a column in the output.
    
    Args:
        cifti_in: the input cifti file.
        cifti_out: the output cifti file.
        limit_gb: restrict memory usage\
            \
            memory limit in gigabytes.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiTransposeOutputs`).
    """
    params = cifti_transpose_params(
        cifti_in=cifti_in,
        cifti_out=cifti_out,
        limit_gb=limit_gb,
    )
    return cifti_transpose_execute(params, runner)


__all__ = [
    "CIFTI_TRANSPOSE_METADATA",
    "CiftiTransposeOutputs",
    "CiftiTransposeParamsDict",
    "CiftiTransposeParamsDictTagged",
    "cifti_transpose",
    "cifti_transpose_execute",
    "cifti_transpose_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_SET_COORDINATES_METADATA = Metadata(
    id="6300d8d0bb2424f11e6f8bc0f67e98142564cdff.workbench",
    name="surface-set-coordinates",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceSetCoordinatesParamsDictNoTag = typing.TypedDict('_SurfaceSetCoordinatesParamsDictNoTag', {
    "surface-in": InputPathType,
    "coord-metric": InputPathType,
    "surface-out": str,
})
SurfaceSetCoordinatesParamsDictTagged = typing.TypedDict('SurfaceSetCoordinatesParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-set-coordinates"],
    "surface-in": InputPathType,
    "coord-metric": InputPathType,
    "surface-out": str,
})
SurfaceSetCoordinatesParamsDict = _SurfaceSetCoordinatesParamsDictNoTag | SurfaceSetCoordinatesParamsDictTagged


class SurfaceSetCoordinatesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceSetCoordinatesParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """the new surface"""


def surface_set_coordinates_params(
    surface_in: InputPathType,
    coord_metric: InputPathType,
    surface_out: str,
) -> SurfaceSetCoordinatesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface_in: the surface to use for the topology.
        coord_metric: the new coordinates, as a 3-column metric file.
        surface_out: the new surface.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-set-coordinates",
        "surface-in": surface_in,
        "coord-metric": coord_metric,
        "surface-out": surface_out,
    }
    return params


def surface_set_coordinates_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceSetCoordinatesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface-in", None) is None:
        raise StyxValidationError("`surface-in` must not be None")
    if not isinstance(params["surface-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-in` has the wrong type: Received `{type(params.get("surface-in", None))}` expected `InputPathType`')
    if params.get("coord-metric", None) is None:
        raise StyxValidationError("`coord-metric` must not be None")
    if not isinstance(params["coord-metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`coord-metric` has the wrong type: Received `{type(params.get("coord-metric", None))}` expected `InputPathType`')
    if params.get("surface-out", None) is None:
        raise StyxValidationError("`surface-out` must not be None")
    if not isinstance(params["surface-out"], str):
        raise StyxValidationError(f'`surface-out` has the wrong type: Received `{type(params.get("surface-out", None))}` expected `str`')


def surface_set_coordinates_cargs(
    params: SurfaceSetCoordinatesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-set-coordinates"
    ])
    cargs.append(execution.input_file(params.get("surface-in", None)))
    cargs.append(execution.input_file(params.get("coord-metric", None)))
    cargs.append(params.get("surface-out", None))
    return cargs


def surface_set_coordinates_outputs(
    params: SurfaceSetCoordinatesParamsDict,
    execution: Execution,
) -> SurfaceSetCoordinatesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceSetCoordinatesOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface-out", None)),
    )
    return ret


def surface_set_coordinates_execute(
    params: SurfaceSetCoordinatesParamsDict,
    runner: Runner | None = None,
) -> SurfaceSetCoordinatesOutputs:
    """
    MODIFY COORDINATES OF A SURFACE.
    
    Takes the topology from an existing surface file, and uses values from a
    metric file as coordinates to construct a new surface file.
    
    See -surface-coordinates-to-metric for how to get surface coordinates as a
    metric file, such that you can then modify them via metric commands, etc.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceSetCoordinatesOutputs`).
    """
    surface_set_coordinates_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_SET_COORDINATES_METADATA)
    params = execution.params(params)
    cargs = surface_set_coordinates_cargs(params, execution)
    ret = surface_set_coordinates_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_set_coordinates(
    surface_in: InputPathType,
    coord_metric: InputPathType,
    surface_out: str,
    runner: Runner | None = None,
) -> SurfaceSetCoordinatesOutputs:
    """
    MODIFY COORDINATES OF A SURFACE.
    
    Takes the topology from an existing surface file, and uses values from a
    metric file as coordinates to construct a new surface file.
    
    See -surface-coordinates-to-metric for how to get surface coordinates as a
    metric file, such that you can then modify them via metric commands, etc.
    
    Args:
        surface_in: the surface to use for the topology.
        coord_metric: the new coordinates, as a 3-column metric file.
        surface_out: the new surface.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceSetCoordinatesOutputs`).
    """
    params = surface_set_coordinates_params(
        surface_in=surface_in,
        coord_metric=coord_metric,
        surface_out=surface_out,
    )
    return surface_set_coordinates_execute(params, runner)


__all__ = [
    "SURFACE_SET_COORDINATES_METADATA",
    "SurfaceSetCoordinatesOutputs",
    "SurfaceSetCoordinatesParamsDict",
    "SurfaceSetCoordinatesParamsDictTagged",
    "surface_set_coordinates",
    "surface_set_coordinates_execute",
    "surface_set_coordinates_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_RESTRICT_DENSE_MAP_METADATA = Metadata(
    id="15d326c1749b72cd350d3064369db0c366e4f1b4.workbench",
    name="cifti-restrict-dense-map",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiRestrictDenseMapParamsDictNoTag = typing.TypedDict('_CiftiRestrictDenseMapParamsDictNoTag', {
    "cifti-in": InputPathType,
    "direction": str,
    "cifti-out": str,
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
})
CiftiRestrictDenseMapParamsDictTagged = typing.TypedDict('CiftiRestrictDenseMapParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-restrict-dense-map"],
    "cifti-in": InputPathType,
    "direction": str,
    "cifti-out": str,
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
})
CiftiRestrictDenseMapParamsDict = _CiftiRestrictDenseMapParamsDictNoTag | CiftiRestrictDenseMapParamsDictTagged


class CiftiRestrictDenseMapOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiRestrictDenseMapParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti"""


def cifti_restrict_dense_map_params(
    cifti_in: InputPathType,
    direction: str,
    cifti_out: str,
    roi_vol: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    roi_metric_2: InputPathType | None = None,
    roi_cifti: InputPathType | None = None,
) -> CiftiRestrictDenseMapParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti.
        direction: which dimension to change the mapping on (integer, 'ROW', or\
            'COLUMN').
        cifti_out: the output cifti.
        roi_vol: voxels to use\
            \
            the roi volume file.
        roi_metric: vertices to use from cerebellum\
            \
            the cerebellum roi as a metric file.
        roi_metric_: vertices to use from right hemisphere\
            \
            the right roi as a metric file.
        roi_metric_2: vertices to use from left hemisphere\
            \
            the left roi as a metric file.
        roi_cifti: cifti file containing combined rois\
            \
            the rois as a cifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-restrict-dense-map",
        "cifti-in": cifti_in,
        "direction": direction,
        "cifti-out": cifti_out,
    }
    if roi_vol is not None:
        params["roi-vol"] = roi_vol
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    if roi_metric_ is not None:
        params["roi-metric"] = roi_metric_
    if roi_metric_2 is not None:
        params["roi-metric"] = roi_metric_2
    if roi_cifti is not None:
        params["roi-cifti"] = roi_cifti
    return params


def cifti_restrict_dense_map_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiRestrictDenseMapParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("roi-vol", None) is not None:
        if not isinstance(params["roi-vol"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-vol` has the wrong type: Received `{type(params.get("roi-vol", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-cifti", None) is not None:
        if not isinstance(params["roi-cifti"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-cifti` has the wrong type: Received `{type(params.get("roi-cifti", None))}` expected `InputPathType | None`')


def cifti_restrict_dense_map_cargs(
    params: CiftiRestrictDenseMapParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-restrict-dense-map"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("roi-vol", None) is not None:
        cargs.extend([
            "-vol-roi",
            execution.input_file(params.get("roi-vol", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-cerebellum-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-right-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-left-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-cifti", None) is not None:
        cargs.extend([
            "-cifti-roi",
            execution.input_file(params.get("roi-cifti", None))
        ])
    return cargs


def cifti_restrict_dense_map_outputs(
    params: CiftiRestrictDenseMapParamsDict,
    execution: Execution,
) -> CiftiRestrictDenseMapOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiRestrictDenseMapOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_restrict_dense_map_execute(
    params: CiftiRestrictDenseMapParamsDict,
    runner: Runner | None = None,
) -> CiftiRestrictDenseMapOutputs:
    """
    EXCLUDE BRAINORDINATES FROM A CIFTI FILE.
    
    Writes a modified version of <cifti-in>, where all brainordinates outside
    the specified roi(s) are removed from the file. The direction can be either
    an integer starting from 1, or the strings 'ROW' or 'COLUMN'. If -cifti-roi
    is specified, no other -*-roi option may be specified. If not using
    -cifti-roi, any -*-roi options not present will discard the relevant
    structure, if present in the input file.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiRestrictDenseMapOutputs`).
    """
    cifti_restrict_dense_map_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_RESTRICT_DENSE_MAP_METADATA)
    params = execution.params(params)
    cargs = cifti_restrict_dense_map_cargs(params, execution)
    ret = cifti_restrict_dense_map_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_restrict_dense_map(
    cifti_in: InputPathType,
    direction: str,
    cifti_out: str,
    roi_vol: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    roi_metric_2: InputPathType | None = None,
    roi_cifti: InputPathType | None = None,
    runner: Runner | None = None,
) -> CiftiRestrictDenseMapOutputs:
    """
    EXCLUDE BRAINORDINATES FROM A CIFTI FILE.
    
    Writes a modified version of <cifti-in>, where all brainordinates outside
    the specified roi(s) are removed from the file. The direction can be either
    an integer starting from 1, or the strings 'ROW' or 'COLUMN'. If -cifti-roi
    is specified, no other -*-roi option may be specified. If not using
    -cifti-roi, any -*-roi options not present will discard the relevant
    structure, if present in the input file.
    
    Args:
        cifti_in: the input cifti.
        direction: which dimension to change the mapping on (integer, 'ROW', or\
            'COLUMN').
        cifti_out: the output cifti.
        roi_vol: voxels to use\
            \
            the roi volume file.
        roi_metric: vertices to use from cerebellum\
            \
            the cerebellum roi as a metric file.
        roi_metric_: vertices to use from right hemisphere\
            \
            the right roi as a metric file.
        roi_metric_2: vertices to use from left hemisphere\
            \
            the left roi as a metric file.
        roi_cifti: cifti file containing combined rois\
            \
            the rois as a cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiRestrictDenseMapOutputs`).
    """
    params = cifti_restrict_dense_map_params(
        cifti_in=cifti_in,
        direction=direction,
        cifti_out=cifti_out,
        roi_vol=roi_vol,
        roi_metric=roi_metric,
        roi_metric_=roi_metric_,
        roi_metric_2=roi_metric_2,
        roi_cifti=roi_cifti,
    )
    return cifti_restrict_dense_map_execute(params, runner)


__all__ = [
    "CIFTI_RESTRICT_DENSE_MAP_METADATA",
    "CiftiRestrictDenseMapOutputs",
    "CiftiRestrictDenseMapParamsDict",
    "CiftiRestrictDenseMapParamsDictTagged",
    "cifti_restrict_dense_map",
    "cifti_restrict_dense_map_execute",
    "cifti_restrict_dense_map_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_TFCE_METADATA = Metadata(
    id="08ff0e7a57ecc93cc6a3d034347b01fa26e0507a.workbench",
    name="metric-tfce",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricTfcePresmoothParamsDictNoTag = typing.TypedDict('_MetricTfcePresmoothParamsDictNoTag', {
    "kernel": float,
    "fwhm": bool,
})
MetricTfcePresmoothParamsDictTagged = typing.TypedDict('MetricTfcePresmoothParamsDictTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "fwhm": bool,
})
MetricTfcePresmoothParamsDict = _MetricTfcePresmoothParamsDictNoTag | MetricTfcePresmoothParamsDictTagged


_MetricTfceParametersParamsDictNoTag = typing.TypedDict('_MetricTfceParametersParamsDictNoTag', {
    "E": float,
    "H": float,
})
MetricTfceParametersParamsDictTagged = typing.TypedDict('MetricTfceParametersParamsDictTagged', {
    "@type": typing.Literal["parameters"],
    "E": float,
    "H": float,
})
MetricTfceParametersParamsDict = _MetricTfceParametersParamsDictNoTag | MetricTfceParametersParamsDictTagged


_MetricTfceParamsDictNoTag = typing.TypedDict('_MetricTfceParamsDictNoTag', {
    "surface": InputPathType,
    "metric-in": InputPathType,
    "metric-out": str,
    "presmooth": typing.NotRequired[MetricTfcePresmoothParamsDict | None],
    "parameters": typing.NotRequired[MetricTfceParametersParamsDict | None],
    "area-metric": typing.NotRequired[InputPathType | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricTfceParamsDictTagged = typing.TypedDict('MetricTfceParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-tfce"],
    "surface": InputPathType,
    "metric-in": InputPathType,
    "metric-out": str,
    "presmooth": typing.NotRequired[MetricTfcePresmoothParamsDict | None],
    "parameters": typing.NotRequired[MetricTfceParametersParamsDict | None],
    "area-metric": typing.NotRequired[InputPathType | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricTfceParamsDict = _MetricTfceParamsDictNoTag | MetricTfceParamsDictTagged


def metric_tfce_presmooth(
    kernel: float,
    fwhm: bool = False,
) -> MetricTfcePresmoothParamsDictTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "fwhm": fwhm,
    }
    return params


def metric_tfce_presmooth_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricTfcePresmoothParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def metric_tfce_presmooth_cargs(
    params: MetricTfcePresmoothParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-presmooth",
        str(params.get("kernel", None))
    ])
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def metric_tfce_parameters(
    e: float,
    h: float,
) -> MetricTfceParametersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        e: exponent for cluster area (default 1.0).
        h: exponent for threshold value (default 2.0).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "parameters",
        "E": e,
        "H": h,
    }
    return params


def metric_tfce_parameters_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricTfceParametersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("E", None) is None:
        raise StyxValidationError("`E` must not be None")
    if not isinstance(params["E"], (float, int)):
        raise StyxValidationError(f'`E` has the wrong type: Received `{type(params.get("E", None))}` expected `float`')
    if params.get("H", None) is None:
        raise StyxValidationError("`H` must not be None")
    if not isinstance(params["H"], (float, int)):
        raise StyxValidationError(f'`H` has the wrong type: Received `{type(params.get("H", None))}` expected `float`')


def metric_tfce_parameters_cargs(
    params: MetricTfceParametersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-parameters",
        str(params.get("E", None)),
        str(params.get("H", None))
    ])
    return cargs


class MetricTfceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricTfceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_tfce_params(
    surface: InputPathType,
    metric_in: InputPathType,
    metric_out: str,
    presmooth: MetricTfcePresmoothParamsDict | None = None,
    parameters: MetricTfceParametersParamsDict | None = None,
    area_metric: InputPathType | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
) -> MetricTfceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to compute on.
        metric_in: the metric to run TFCE on.
        metric_out: the output metric.
        presmooth: smooth the metric before running TFCE.
        parameters: set parameters for TFCE integral.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        column: select a single column\
            \
            the column number or name.
        roi_metric: select a region of interest to run TFCE on\
            \
            the area to run TFCE on, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-tfce",
        "surface": surface,
        "metric-in": metric_in,
        "metric-out": metric_out,
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    if parameters is not None:
        params["parameters"] = parameters
    if area_metric is not None:
        params["area-metric"] = area_metric
    if column is not None:
        params["column"] = column
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def metric_tfce_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricTfceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("presmooth", None) is not None:
        metric_tfce_presmooth_validate(params["presmooth"])
    if params.get("parameters", None) is not None:
        metric_tfce_parameters_validate(params["parameters"])
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def metric_tfce_cargs(
    params: MetricTfceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-tfce"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric-in", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("presmooth", None) is not None or params.get("parameters", None) is not None:
        cargs.extend([
            *(metric_tfce_presmooth_cargs(params.get("presmooth", None), execution) if (params.get("presmooth", None) is not None) else []),
            *(metric_tfce_parameters_cargs(params.get("parameters", None), execution) if (params.get("parameters", None) is not None) else [])
        ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def metric_tfce_outputs(
    params: MetricTfceParamsDict,
    execution: Execution,
) -> MetricTfceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricTfceOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_tfce_execute(
    params: MetricTfceParamsDict,
    runner: Runner | None = None,
) -> MetricTfceOutputs:
    """
    DO TFCE ON A METRIC FILE.
    
    This command does not do any statistical analysis. Please use something like
    PALM if you are just trying to do statistics on your data.
    
    Threshold-free cluster enhancement is a method to increase the relative
    value of regions that would form clusters in a standard thresholding test.
    This is accomplished by evaluating the integral of:
    
    e(h, p)^E * h^H * dh
    
    at each vertex p, where h ranges from 0 to the maximum value in the data,
    and e(h, p) is the extent of the cluster containing vertex p at threshold h.
    Negative values are similarly enhanced by negating the data, running the
    same process, and negating the result.
    
    When using -presmooth with -corrected-areas, note that it is an approximate
    correction within the smoothing algorithm (the TFCE correction is exact).
    Doing smoothing on individual surfaces before averaging/TFCE is preferred,
    when possible, in order to better tie the smoothing kernel size to the
    original feature size.
    
    The TFCE method is explained in: Smith SM, Nichols TE., "Threshold-free
    cluster enhancement: addressing problems of smoothing, threshold dependence
    and localisation in cluster inference." Neuroimage. 2009 Jan 1;44(1):83-98.
    PMID: 18501637.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricTfceOutputs`).
    """
    metric_tfce_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_TFCE_METADATA)
    params = execution.params(params)
    cargs = metric_tfce_cargs(params, execution)
    ret = metric_tfce_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_tfce(
    surface: InputPathType,
    metric_in: InputPathType,
    metric_out: str,
    presmooth: MetricTfcePresmoothParamsDict | None = None,
    parameters: MetricTfceParametersParamsDict | None = None,
    area_metric: InputPathType | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> MetricTfceOutputs:
    """
    DO TFCE ON A METRIC FILE.
    
    This command does not do any statistical analysis. Please use something like
    PALM if you are just trying to do statistics on your data.
    
    Threshold-free cluster enhancement is a method to increase the relative
    value of regions that would form clusters in a standard thresholding test.
    This is accomplished by evaluating the integral of:
    
    e(h, p)^E * h^H * dh
    
    at each vertex p, where h ranges from 0 to the maximum value in the data,
    and e(h, p) is the extent of the cluster containing vertex p at threshold h.
    Negative values are similarly enhanced by negating the data, running the
    same process, and negating the result.
    
    When using -presmooth with -corrected-areas, note that it is an approximate
    correction within the smoothing algorithm (the TFCE correction is exact).
    Doing smoothing on individual surfaces before averaging/TFCE is preferred,
    when possible, in order to better tie the smoothing kernel size to the
    original feature size.
    
    The TFCE method is explained in: Smith SM, Nichols TE., "Threshold-free
    cluster enhancement: addressing problems of smoothing, threshold dependence
    and localisation in cluster inference." Neuroimage. 2009 Jan 1;44(1):83-98.
    PMID: 18501637.
    
    Args:
        surface: the surface to compute on.
        metric_in: the metric to run TFCE on.
        metric_out: the output metric.
        presmooth: smooth the metric before running TFCE.
        parameters: set parameters for TFCE integral.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        column: select a single column\
            \
            the column number or name.
        roi_metric: select a region of interest to run TFCE on\
            \
            the area to run TFCE on, as a metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricTfceOutputs`).
    """
    params = metric_tfce_params(
        surface=surface,
        metric_in=metric_in,
        metric_out=metric_out,
        presmooth=presmooth,
        parameters=parameters,
        area_metric=area_metric,
        column=column,
        roi_metric=roi_metric,
    )
    return metric_tfce_execute(params, runner)


__all__ = [
    "METRIC_TFCE_METADATA",
    "MetricTfceOutputs",
    "MetricTfceParametersParamsDict",
    "MetricTfceParametersParamsDictTagged",
    "MetricTfceParamsDict",
    "MetricTfceParamsDictTagged",
    "MetricTfcePresmoothParamsDict",
    "MetricTfcePresmoothParamsDictTagged",
    "metric_tfce",
    "metric_tfce_execute",
    "metric_tfce_parameters",
    "metric_tfce_params",
    "metric_tfce_presmooth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_FIND_CLUSTERS_METADATA = Metadata(
    id="80ccd8653d75a12acc8bcc204d1a6379b0a31cfe.workbench",
    name="volume-find-clusters",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeFindClustersParamsDictNoTag = typing.TypedDict('_VolumeFindClustersParamsDictNoTag', {
    "volume-in": InputPathType,
    "value-threshold": float,
    "minimum-volume": float,
    "volume-out": str,
    "startval": typing.NotRequired[int | None],
    "distance": typing.NotRequired[float | None],
    "ratio": typing.NotRequired[float | None],
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "less-than": bool,
})
VolumeFindClustersParamsDictTagged = typing.TypedDict('VolumeFindClustersParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-find-clusters"],
    "volume-in": InputPathType,
    "value-threshold": float,
    "minimum-volume": float,
    "volume-out": str,
    "startval": typing.NotRequired[int | None],
    "distance": typing.NotRequired[float | None],
    "ratio": typing.NotRequired[float | None],
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "less-than": bool,
})
VolumeFindClustersParamsDict = _VolumeFindClustersParamsDictNoTag | VolumeFindClustersParamsDictTagged


class VolumeFindClustersOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeFindClustersParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_find_clusters_params(
    volume_in: InputPathType,
    value_threshold: float,
    minimum_volume: float,
    volume_out: str,
    startval: int | None = None,
    distance: float | None = None,
    ratio: float | None = None,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
    less_than: bool = False,
) -> VolumeFindClustersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        value_threshold: threshold for data values.
        minimum_volume: threshold for cluster volume, in mm^3.
        volume_out: the output volume.
        startval: start labeling clusters from a value other than 1\
            \
            the value to give the first cluster found.
        distance: ignore clusters further than a given distance from the\
            largest cluster\
            \
            how far from the largest cluster a cluster can be, edge to edge, in\
            mm.
        ratio: ignore clusters smaller than a given fraction of the largest\
            cluster in map\
            \
            fraction of the largest cluster's volume.
        subvol: select a single subvolume\
            \
            the subvolume number or name.
        roi_volume: select a region of interest\
            \
            the roi, as a volume file.
        less_than: find values less than <value-threshold>, rather than greater.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-find-clusters",
        "volume-in": volume_in,
        "value-threshold": value_threshold,
        "minimum-volume": minimum_volume,
        "volume-out": volume_out,
        "less-than": less_than,
    }
    if startval is not None:
        params["startval"] = startval
    if distance is not None:
        params["distance"] = distance
    if ratio is not None:
        params["ratio"] = ratio
    if subvol is not None:
        params["subvol"] = subvol
    if roi_volume is not None:
        params["roi-volume"] = roi_volume
    return params


def volume_find_clusters_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeFindClustersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("value-threshold", None) is None:
        raise StyxValidationError("`value-threshold` must not be None")
    if not isinstance(params["value-threshold"], (float, int)):
        raise StyxValidationError(f'`value-threshold` has the wrong type: Received `{type(params.get("value-threshold", None))}` expected `float`')
    if params.get("minimum-volume", None) is None:
        raise StyxValidationError("`minimum-volume` must not be None")
    if not isinstance(params["minimum-volume"], (float, int)):
        raise StyxValidationError(f'`minimum-volume` has the wrong type: Received `{type(params.get("minimum-volume", None))}` expected `float`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("startval", None) is not None:
        if not isinstance(params["startval"], int):
            raise StyxValidationError(f'`startval` has the wrong type: Received `{type(params.get("startval", None))}` expected `int | None`')
    if params.get("distance", None) is not None:
        if not isinstance(params["distance"], (float, int)):
            raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float | None`')
    if params.get("ratio", None) is not None:
        if not isinstance(params["ratio"], (float, int)):
            raise StyxValidationError(f'`ratio` has the wrong type: Received `{type(params.get("ratio", None))}` expected `float | None`')
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')
    if params.get("less-than", False) is None:
        raise StyxValidationError("`less-than` must not be None")
    if not isinstance(params["less-than"], bool):
        raise StyxValidationError(f'`less-than` has the wrong type: Received `{type(params.get("less-than", False))}` expected `bool`')


def volume_find_clusters_cargs(
    params: VolumeFindClustersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-find-clusters"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(str(params.get("value-threshold", None)))
    cargs.append(str(params.get("minimum-volume", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("startval", None) is not None:
        cargs.extend([
            "-start",
            str(params.get("startval", None))
        ])
    if params.get("distance", None) is not None:
        cargs.extend([
            "-distance",
            str(params.get("distance", None))
        ])
    if params.get("ratio", None) is not None:
        cargs.extend([
            "-size-ratio",
            str(params.get("ratio", None))
        ])
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    if params.get("less-than", False):
        cargs.append("-less-than")
    return cargs


def volume_find_clusters_outputs(
    params: VolumeFindClustersParamsDict,
    execution: Execution,
) -> VolumeFindClustersOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeFindClustersOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_find_clusters_execute(
    params: VolumeFindClustersParamsDict,
    runner: Runner | None = None,
) -> VolumeFindClustersOutputs:
    """
    FILTER CLUSTERS BY VOLUME.
    
    Outputs a volume with nonzero integers for all voxels within a large enough
    cluster, and zeros elsewhere. The integers denote cluster membership (by
    default, first cluster found will use value 1, second cluster 2, etc).
    Cluster values are not reused across frames of the output, but instead keep
    counting up. By default, values greater than <value-threshold> are
    considered to be in a cluster, use -less-than to test for values less than
    the threshold. To apply this as a mask to the data, or to do more
    complicated thresholding, see -volume-math.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeFindClustersOutputs`).
    """
    volume_find_clusters_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_FIND_CLUSTERS_METADATA)
    params = execution.params(params)
    cargs = volume_find_clusters_cargs(params, execution)
    ret = volume_find_clusters_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_find_clusters(
    volume_in: InputPathType,
    value_threshold: float,
    minimum_volume: float,
    volume_out: str,
    startval: int | None = None,
    distance: float | None = None,
    ratio: float | None = None,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
    less_than: bool = False,
    runner: Runner | None = None,
) -> VolumeFindClustersOutputs:
    """
    FILTER CLUSTERS BY VOLUME.
    
    Outputs a volume with nonzero integers for all voxels within a large enough
    cluster, and zeros elsewhere. The integers denote cluster membership (by
    default, first cluster found will use value 1, second cluster 2, etc).
    Cluster values are not reused across frames of the output, but instead keep
    counting up. By default, values greater than <value-threshold> are
    considered to be in a cluster, use -less-than to test for values less than
    the threshold. To apply this as a mask to the data, or to do more
    complicated thresholding, see -volume-math.
    
    Args:
        volume_in: the input volume.
        value_threshold: threshold for data values.
        minimum_volume: threshold for cluster volume, in mm^3.
        volume_out: the output volume.
        startval: start labeling clusters from a value other than 1\
            \
            the value to give the first cluster found.
        distance: ignore clusters further than a given distance from the\
            largest cluster\
            \
            how far from the largest cluster a cluster can be, edge to edge, in\
            mm.
        ratio: ignore clusters smaller than a given fraction of the largest\
            cluster in map\
            \
            fraction of the largest cluster's volume.
        subvol: select a single subvolume\
            \
            the subvolume number or name.
        roi_volume: select a region of interest\
            \
            the roi, as a volume file.
        less_than: find values less than <value-threshold>, rather than greater.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeFindClustersOutputs`).
    """
    params = volume_find_clusters_params(
        volume_in=volume_in,
        value_threshold=value_threshold,
        minimum_volume=minimum_volume,
        volume_out=volume_out,
        startval=startval,
        distance=distance,
        ratio=ratio,
        subvol=subvol,
        roi_volume=roi_volume,
        less_than=less_than,
    )
    return volume_find_clusters_execute(params, runner)


__all__ = [
    "VOLUME_FIND_CLUSTERS_METADATA",
    "VolumeFindClustersOutputs",
    "VolumeFindClustersParamsDict",
    "VolumeFindClustersParamsDictTagged",
    "volume_find_clusters",
    "volume_find_clusters_execute",
    "volume_find_clusters_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_APPLY_AFFINE_METADATA = Metadata(
    id="248f32501c3003298999c7da69bba8ad26520e39.workbench",
    name="surface-apply-affine",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceApplyAffineFlirtParamsDictNoTag = typing.TypedDict('_SurfaceApplyAffineFlirtParamsDictNoTag', {
    "source-volume": str,
    "target-volume": str,
})
SurfaceApplyAffineFlirtParamsDictTagged = typing.TypedDict('SurfaceApplyAffineFlirtParamsDictTagged', {
    "@type": typing.Literal["flirt"],
    "source-volume": str,
    "target-volume": str,
})
SurfaceApplyAffineFlirtParamsDict = _SurfaceApplyAffineFlirtParamsDictNoTag | SurfaceApplyAffineFlirtParamsDictTagged


_SurfaceApplyAffineParamsDictNoTag = typing.TypedDict('_SurfaceApplyAffineParamsDictNoTag', {
    "in-surf": InputPathType,
    "affine": str,
    "out-surf": str,
    "flirt": typing.NotRequired[SurfaceApplyAffineFlirtParamsDict | None],
})
SurfaceApplyAffineParamsDictTagged = typing.TypedDict('SurfaceApplyAffineParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-apply-affine"],
    "in-surf": InputPathType,
    "affine": str,
    "out-surf": str,
    "flirt": typing.NotRequired[SurfaceApplyAffineFlirtParamsDict | None],
})
SurfaceApplyAffineParamsDict = _SurfaceApplyAffineParamsDictNoTag | SurfaceApplyAffineParamsDictTagged


def surface_apply_affine_flirt(
    source_volume: str,
    target_volume: str,
) -> SurfaceApplyAffineFlirtParamsDictTagged:
    """
    Build parameters.
    
    Args:
        source_volume: the source volume used when generating the affine.
        target_volume: the target volume used when generating the affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "flirt",
        "source-volume": source_volume,
        "target-volume": target_volume,
    }
    return params


def surface_apply_affine_flirt_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceApplyAffineFlirtParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("source-volume", None) is None:
        raise StyxValidationError("`source-volume` must not be None")
    if not isinstance(params["source-volume"], str):
        raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str`')
    if params.get("target-volume", None) is None:
        raise StyxValidationError("`target-volume` must not be None")
    if not isinstance(params["target-volume"], str):
        raise StyxValidationError(f'`target-volume` has the wrong type: Received `{type(params.get("target-volume", None))}` expected `str`')


def surface_apply_affine_flirt_cargs(
    params: SurfaceApplyAffineFlirtParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-flirt",
        params.get("source-volume", None),
        params.get("target-volume", None)
    ])
    return cargs


class SurfaceApplyAffineOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceApplyAffineParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_surf: OutputPathType
    """the output transformed surface"""


def surface_apply_affine_params(
    in_surf: InputPathType,
    affine: str,
    out_surf: str,
    flirt: SurfaceApplyAffineFlirtParamsDict | None = None,
) -> SurfaceApplyAffineParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_surf: the surface to transform.
        affine: the affine file.
        out_surf: the output transformed surface.
        flirt: MUST be used if affine is a flirt affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-apply-affine",
        "in-surf": in_surf,
        "affine": affine,
        "out-surf": out_surf,
    }
    if flirt is not None:
        params["flirt"] = flirt
    return params


def surface_apply_affine_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceApplyAffineParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("in-surf", None) is None:
        raise StyxValidationError("`in-surf` must not be None")
    if not isinstance(params["in-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in-surf` has the wrong type: Received `{type(params.get("in-surf", None))}` expected `InputPathType`')
    if params.get("affine", None) is None:
        raise StyxValidationError("`affine` must not be None")
    if not isinstance(params["affine"], str):
        raise StyxValidationError(f'`affine` has the wrong type: Received `{type(params.get("affine", None))}` expected `str`')
    if params.get("out-surf", None) is None:
        raise StyxValidationError("`out-surf` must not be None")
    if not isinstance(params["out-surf"], str):
        raise StyxValidationError(f'`out-surf` has the wrong type: Received `{type(params.get("out-surf", None))}` expected `str`')
    if params.get("flirt", None) is not None:
        surface_apply_affine_flirt_validate(params["flirt"])


def surface_apply_affine_cargs(
    params: SurfaceApplyAffineParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-apply-affine"
    ])
    cargs.append(execution.input_file(params.get("in-surf", None)))
    cargs.append(params.get("affine", None))
    cargs.append(params.get("out-surf", None))
    if params.get("flirt", None) is not None:
        cargs.extend(surface_apply_affine_flirt_cargs(params.get("flirt", None), execution))
    return cargs


def surface_apply_affine_outputs(
    params: SurfaceApplyAffineParamsDict,
    execution: Execution,
) -> SurfaceApplyAffineOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceApplyAffineOutputs(
        root=execution.output_file("."),
        out_surf=execution.output_file(params.get("out-surf", None)),
    )
    return ret


def surface_apply_affine_execute(
    params: SurfaceApplyAffineParamsDict,
    runner: Runner | None = None,
) -> SurfaceApplyAffineOutputs:
    """
    APPLY AFFINE TRANSFORM TO SURFACE FILE.
    
    For flirt matrices, you must use the -flirt option, because flirt matrices
    are not a complete description of the coordinate transform they represent.
    If the -flirt option is not present, the affine must be a nifti 'world'
    affine, which can be obtained with the -convert-affine command, or aff_conv
    from the 4dfp suite.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceApplyAffineOutputs`).
    """
    surface_apply_affine_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_APPLY_AFFINE_METADATA)
    params = execution.params(params)
    cargs = surface_apply_affine_cargs(params, execution)
    ret = surface_apply_affine_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_apply_affine(
    in_surf: InputPathType,
    affine: str,
    out_surf: str,
    flirt: SurfaceApplyAffineFlirtParamsDict | None = None,
    runner: Runner | None = None,
) -> SurfaceApplyAffineOutputs:
    """
    APPLY AFFINE TRANSFORM TO SURFACE FILE.
    
    For flirt matrices, you must use the -flirt option, because flirt matrices
    are not a complete description of the coordinate transform they represent.
    If the -flirt option is not present, the affine must be a nifti 'world'
    affine, which can be obtained with the -convert-affine command, or aff_conv
    from the 4dfp suite.
    
    Args:
        in_surf: the surface to transform.
        affine: the affine file.
        out_surf: the output transformed surface.
        flirt: MUST be used if affine is a flirt affine.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceApplyAffineOutputs`).
    """
    params = surface_apply_affine_params(
        in_surf=in_surf,
        affine=affine,
        out_surf=out_surf,
        flirt=flirt,
    )
    return surface_apply_affine_execute(params, runner)


__all__ = [
    "SURFACE_APPLY_AFFINE_METADATA",
    "SurfaceApplyAffineFlirtParamsDict",
    "SurfaceApplyAffineFlirtParamsDictTagged",
    "SurfaceApplyAffineOutputs",
    "SurfaceApplyAffineParamsDict",
    "SurfaceApplyAffineParamsDictTagged",
    "surface_apply_affine",
    "surface_apply_affine_execute",
    "surface_apply_affine_flirt",
    "surface_apply_affine_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FOCI_GET_PROJECTION_VERTEX_METADATA = Metadata(
    id="9698c37ae8d619d4e10d7f0fad13430ab0600e96.workbench",
    name="foci-get-projection-vertex",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_FociGetProjectionVertexParamsDictNoTag = typing.TypedDict('_FociGetProjectionVertexParamsDictNoTag', {
    "foci": InputPathType,
    "surface": InputPathType,
    "metric-out": str,
    "name": typing.NotRequired[str | None],
})
FociGetProjectionVertexParamsDictTagged = typing.TypedDict('FociGetProjectionVertexParamsDictTagged', {
    "@type": typing.Literal["workbench/foci-get-projection-vertex"],
    "foci": InputPathType,
    "surface": InputPathType,
    "metric-out": str,
    "name": typing.NotRequired[str | None],
})
FociGetProjectionVertexParamsDict = _FociGetProjectionVertexParamsDictNoTag | FociGetProjectionVertexParamsDictTagged


class FociGetProjectionVertexOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FociGetProjectionVertexParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric file"""


def foci_get_projection_vertex_params(
    foci: InputPathType,
    surface: InputPathType,
    metric_out: str,
    name: str | None = None,
) -> FociGetProjectionVertexParamsDictTagged:
    """
    Build parameters.
    
    Args:
        foci: the foci file.
        surface: the surface related to the foci file.
        metric_out: the output metric file.
        name: select a focus by name\
            \
            the name of the focus.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/foci-get-projection-vertex",
        "foci": foci,
        "surface": surface,
        "metric-out": metric_out,
    }
    if name is not None:
        params["name"] = name
    return params


def foci_get_projection_vertex_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FociGetProjectionVertexParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("foci", None) is None:
        raise StyxValidationError("`foci` must not be None")
    if not isinstance(params["foci"], (pathlib.Path, str)):
        raise StyxValidationError(f'`foci` has the wrong type: Received `{type(params.get("foci", None))}` expected `InputPathType`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("name", None) is not None:
        if not isinstance(params["name"], str):
            raise StyxValidationError(f'`name` has the wrong type: Received `{type(params.get("name", None))}` expected `str | None`')


def foci_get_projection_vertex_cargs(
    params: FociGetProjectionVertexParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-foci-get-projection-vertex"
    ])
    cargs.append(execution.input_file(params.get("foci", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("name", None) is not None:
        cargs.extend([
            "-name",
            params.get("name", None)
        ])
    return cargs


def foci_get_projection_vertex_outputs(
    params: FociGetProjectionVertexParamsDict,
    execution: Execution,
) -> FociGetProjectionVertexOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FociGetProjectionVertexOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def foci_get_projection_vertex_execute(
    params: FociGetProjectionVertexParamsDict,
    runner: Runner | None = None,
) -> FociGetProjectionVertexOutputs:
    """
    GET PROJECTION VERTEX FOR FOCI.
    
    For each focus, a column is created in <metric-out>, and the vertex with the
    most influence on its projection is assigned a value of 1 in that column,
    with all other vertices 0. If -name is used, only one focus will be used.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FociGetProjectionVertexOutputs`).
    """
    foci_get_projection_vertex_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOCI_GET_PROJECTION_VERTEX_METADATA)
    params = execution.params(params)
    cargs = foci_get_projection_vertex_cargs(params, execution)
    ret = foci_get_projection_vertex_outputs(params, execution)
    execution.run(cargs)
    return ret


def foci_get_projection_vertex(
    foci: InputPathType,
    surface: InputPathType,
    metric_out: str,
    name: str | None = None,
    runner: Runner | None = None,
) -> FociGetProjectionVertexOutputs:
    """
    GET PROJECTION VERTEX FOR FOCI.
    
    For each focus, a column is created in <metric-out>, and the vertex with the
    most influence on its projection is assigned a value of 1 in that column,
    with all other vertices 0. If -name is used, only one focus will be used.
    
    Args:
        foci: the foci file.
        surface: the surface related to the foci file.
        metric_out: the output metric file.
        name: select a focus by name\
            \
            the name of the focus.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FociGetProjectionVertexOutputs`).
    """
    params = foci_get_projection_vertex_params(
        foci=foci,
        surface=surface,
        metric_out=metric_out,
        name=name,
    )
    return foci_get_projection_vertex_execute(params, runner)


__all__ = [
    "FOCI_GET_PROJECTION_VERTEX_METADATA",
    "FociGetProjectionVertexOutputs",
    "FociGetProjectionVertexParamsDict",
    "FociGetProjectionVertexParamsDictTagged",
    "foci_get_projection_vertex",
    "foci_get_projection_vertex_execute",
    "foci_get_projection_vertex_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_CONVERT_METADATA = Metadata(
    id="4df60d9d341bdad2b06cad65892590415be630ec.workbench",
    name="metric-convert",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricConvertToNiftiParamsDictNoTag = typing.TypedDict('_MetricConvertToNiftiParamsDictNoTag', {
    "metric-in": InputPathType,
    "nifti-out": str,
})
MetricConvertToNiftiParamsDictTagged = typing.TypedDict('MetricConvertToNiftiParamsDictTagged', {
    "@type": typing.Literal["to-nifti"],
    "metric-in": InputPathType,
    "nifti-out": str,
})
MetricConvertToNiftiParamsDict = _MetricConvertToNiftiParamsDictNoTag | MetricConvertToNiftiParamsDictTagged


_MetricConvertFromNiftiParamsDictNoTag = typing.TypedDict('_MetricConvertFromNiftiParamsDictNoTag', {
    "nifti-in": InputPathType,
    "surface-in": InputPathType,
    "metric-out": str,
})
MetricConvertFromNiftiParamsDictTagged = typing.TypedDict('MetricConvertFromNiftiParamsDictTagged', {
    "@type": typing.Literal["from-nifti"],
    "nifti-in": InputPathType,
    "surface-in": InputPathType,
    "metric-out": str,
})
MetricConvertFromNiftiParamsDict = _MetricConvertFromNiftiParamsDictNoTag | MetricConvertFromNiftiParamsDictTagged


_MetricConvertParamsDictNoTag = typing.TypedDict('_MetricConvertParamsDictNoTag', {
    "to-nifti": typing.NotRequired[MetricConvertToNiftiParamsDict | None],
    "from-nifti": typing.NotRequired[MetricConvertFromNiftiParamsDict | None],
})
MetricConvertParamsDictTagged = typing.TypedDict('MetricConvertParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-convert"],
    "to-nifti": typing.NotRequired[MetricConvertToNiftiParamsDict | None],
    "from-nifti": typing.NotRequired[MetricConvertFromNiftiParamsDict | None],
})
MetricConvertParamsDict = _MetricConvertParamsDictNoTag | MetricConvertParamsDictTagged


class MetricConvertToNiftiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricConvertToNiftiParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    nifti_out: OutputPathType
    """the output nifti file"""


def metric_convert_to_nifti(
    metric_in: InputPathType,
    nifti_out: str,
) -> MetricConvertToNiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric_in: the metric to convert.
        nifti_out: the output nifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "to-nifti",
        "metric-in": metric_in,
        "nifti-out": nifti_out,
    }
    return params


def metric_convert_to_nifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricConvertToNiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("nifti-out", None) is None:
        raise StyxValidationError("`nifti-out` must not be None")
    if not isinstance(params["nifti-out"], str):
        raise StyxValidationError(f'`nifti-out` has the wrong type: Received `{type(params.get("nifti-out", None))}` expected `str`')


def metric_convert_to_nifti_cargs(
    params: MetricConvertToNiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-to-nifti",
        execution.input_file(params.get("metric-in", None)),
        params.get("nifti-out", None)
    ])
    return cargs


def metric_convert_to_nifti_outputs(
    params: MetricConvertToNiftiParamsDict,
    execution: Execution,
) -> MetricConvertToNiftiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricConvertToNiftiOutputs(
        root=execution.output_file("."),
        nifti_out=execution.output_file(params.get("nifti-out", None)),
    )
    return ret


class MetricConvertFromNiftiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricConvertFromNiftiParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric file"""


def metric_convert_from_nifti(
    nifti_in: InputPathType,
    surface_in: InputPathType,
    metric_out: str,
) -> MetricConvertFromNiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        nifti_in: the nifti file to convert.
        surface_in: surface file to use number of vertices and structure from.
        metric_out: the output metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from-nifti",
        "nifti-in": nifti_in,
        "surface-in": surface_in,
        "metric-out": metric_out,
    }
    return params


def metric_convert_from_nifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricConvertFromNiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("nifti-in", None) is None:
        raise StyxValidationError("`nifti-in` must not be None")
    if not isinstance(params["nifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`nifti-in` has the wrong type: Received `{type(params.get("nifti-in", None))}` expected `InputPathType`')
    if params.get("surface-in", None) is None:
        raise StyxValidationError("`surface-in` must not be None")
    if not isinstance(params["surface-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-in` has the wrong type: Received `{type(params.get("surface-in", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')


def metric_convert_from_nifti_cargs(
    params: MetricConvertFromNiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-from-nifti",
        execution.input_file(params.get("nifti-in", None)),
        execution.input_file(params.get("surface-in", None)),
        params.get("metric-out", None)
    ])
    return cargs


def metric_convert_from_nifti_outputs(
    params: MetricConvertFromNiftiParamsDict,
    execution: Execution,
) -> MetricConvertFromNiftiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricConvertFromNiftiOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


class MetricConvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricConvertParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    to_nifti: MetricConvertToNiftiOutputs | None
    """Outputs from `metric_convert_to_nifti_outputs`."""
    from_nifti: MetricConvertFromNiftiOutputs | None
    """Outputs from `metric_convert_from_nifti_outputs`."""


def metric_convert_params(
    to_nifti: MetricConvertToNiftiParamsDict | None = None,
    from_nifti: MetricConvertFromNiftiParamsDict | None = None,
) -> MetricConvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        to_nifti: convert metric to nifti.
        from_nifti: convert nifti to metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-convert",
    }
    if to_nifti is not None:
        params["to-nifti"] = to_nifti
    if from_nifti is not None:
        params["from-nifti"] = from_nifti
    return params


def metric_convert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricConvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("to-nifti", None) is not None:
        metric_convert_to_nifti_validate(params["to-nifti"])
    if params.get("from-nifti", None) is not None:
        metric_convert_from_nifti_validate(params["from-nifti"])


def metric_convert_cargs(
    params: MetricConvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-convert"
    ])
    if params.get("to-nifti", None) is not None or params.get("from-nifti", None) is not None:
        cargs.extend([
            *(metric_convert_to_nifti_cargs(params.get("to-nifti", None), execution) if (params.get("to-nifti", None) is not None) else []),
            *(metric_convert_from_nifti_cargs(params.get("from-nifti", None), execution) if (params.get("from-nifti", None) is not None) else [])
        ])
    return cargs


def metric_convert_outputs(
    params: MetricConvertParamsDict,
    execution: Execution,
) -> MetricConvertOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricConvertOutputs(
        root=execution.output_file("."),
        to_nifti=metric_convert_to_nifti_outputs(params.get("to-nifti"), execution) if params.get("to-nifti") else None,
        from_nifti=metric_convert_from_nifti_outputs(params.get("from-nifti"), execution) if params.get("from-nifti") else None,
    )
    return ret


def metric_convert_execute(
    params: MetricConvertParamsDict,
    runner: Runner | None = None,
) -> MetricConvertOutputs:
    """
    CONVERT METRIC FILE TO FAKE NIFTI.
    
    The purpose of this command is to convert between metric files and nifti1 so
    that gifti-unaware programs can operate on the data. You must specify
    exactly one of the options.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricConvertOutputs`).
    """
    metric_convert_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_CONVERT_METADATA)
    params = execution.params(params)
    cargs = metric_convert_cargs(params, execution)
    ret = metric_convert_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_convert(
    to_nifti: MetricConvertToNiftiParamsDict | None = None,
    from_nifti: MetricConvertFromNiftiParamsDict | None = None,
    runner: Runner | None = None,
) -> MetricConvertOutputs:
    """
    CONVERT METRIC FILE TO FAKE NIFTI.
    
    The purpose of this command is to convert between metric files and nifti1 so
    that gifti-unaware programs can operate on the data. You must specify
    exactly one of the options.
    
    Args:
        to_nifti: convert metric to nifti.
        from_nifti: convert nifti to metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricConvertOutputs`).
    """
    params = metric_convert_params(
        to_nifti=to_nifti,
        from_nifti=from_nifti,
    )
    return metric_convert_execute(params, runner)


__all__ = [
    "METRIC_CONVERT_METADATA",
    "MetricConvertFromNiftiOutputs",
    "MetricConvertFromNiftiParamsDict",
    "MetricConvertFromNiftiParamsDictTagged",
    "MetricConvertOutputs",
    "MetricConvertParamsDict",
    "MetricConvertParamsDictTagged",
    "MetricConvertToNiftiOutputs",
    "MetricConvertToNiftiParamsDict",
    "MetricConvertToNiftiParamsDictTagged",
    "metric_convert",
    "metric_convert_execute",
    "metric_convert_from_nifti",
    "metric_convert_params",
    "metric_convert_to_nifti",
]
